@@ -37,11 +37,33 @@
 #include <arvgcinteger.h>
 #include <arvgcfloat.h>
 #include <arvgcenumeration.h>
+#include <arvgcenumentry.h>
 #include <arvgcstring.h>
 #include <arvgc.h>
 #include <arvdevice.h>
 #include <arvenums.h>
 
+/**
+ * ArvCameraVendor:
+ * @ARV_CAMERA_VENDOR_UNKNOWN: unknown camera vendor
+ * @ARV_CAMERA_VENDOR_BASLER: Basler
+ * @ARV_CAMERA_VENDOR_PROSILICA: Prosilica
+ */
+
+typedef enum {
+	ARV_CAMERA_VENDOR_UNKNOWN,
+	ARV_CAMERA_VENDOR_BASLER,
+	ARV_CAMERA_VENDOR_PROSILICA
+} ArvCameraVendor;
+
+typedef enum {
+	ARV_CAMERA_SERIES_UNKNOWN,
+	ARV_CAMERA_SERIES_BASLER_ACE,
+	ARV_CAMERA_SERIES_BASLER_SCOUT,
+	ARV_CAMERA_SERIES_BASLER_OTHER,
+	ARV_CAMERA_SERIES_PROSILICA_OTHER
+} ArvCameraSeries;
+
 static GObjectClass *parent_class = NULL;
 
 struct _ArvCameraPrivate {
@@ -49,6 +71,7 @@ struct _ArvCameraPrivate {
 	ArvGc *genicam;
 
 	ArvCameraVendor vendor;
+	ArvCameraSeries series;
 };
 
 /**
@@ -147,13 +170,13 @@ arv_camera_get_sensor_size (ArvCamera *camera, gint *width, gint *height)
  */
 
 void
-arv_camera_set_region (ArvCamera *camera, int x, int y, int width, int height)
+arv_camera_set_region (ArvCamera *camera, gint x, gint y, gint width, gint height)
 {
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 
-	if (x > 0)
+	if (x >= 0)
 		arv_device_set_integer_feature_value (camera->priv->device, "OffsetX", x);
-	if (y > 0)
+	if (y >= 0)
 		arv_device_set_integer_feature_value (camera->priv->device, "OffsetY", y);
 	if (width > 0)
 		arv_device_set_integer_feature_value (camera->priv->device, "Width", width);
@@ -187,6 +210,36 @@ arv_camera_get_region (ArvCamera *camera, gint *x, gint *y, gint *width, gint *h
 		*height = arv_device_get_integer_feature_value (camera->priv->device, "Height");
 }
 
+void
+arv_camera_get_width_bounds (ArvCamera *camera, gint *min, gint *max)
+{
+	gint64 min64, max64;
+
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	arv_device_get_integer_feature_bounds (camera->priv->device, "Width", &min64, &max64);
+
+	if (min != NULL)
+		*min = min64;
+	if (max != NULL)
+		*max = max64;
+}
+
+void
+arv_camera_get_height_bounds (ArvCamera *camera, gint *min, gint *max)
+{
+	gint64 min64, max64;
+
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	arv_device_get_integer_feature_bounds (camera->priv->device, "Height", &min64, &max64);
+
+	if (min != NULL)
+		*min = min64;
+	if (max != NULL)
+		*max = max64;
+}
+
 /**
  * arv_camera_set_binning:
  * @camera: a #ArvCamera
@@ -259,6 +312,33 @@ arv_camera_get_pixel_format (ArvCamera *camera)
 	return arv_device_get_integer_feature_value (camera->priv->device, "PixelFormat");
 }
 
+ArvPixelFormat *
+arv_camera_get_available_pixel_formats (ArvCamera *camera, guint *n_pixel_formats)
+{
+	ArvGcNode *enumeration;
+	ArvPixelFormat *pixel_formats;
+	const GSList *entries, *iter;
+	unsigned int i;
+
+	g_return_val_if_fail (ARV_IS_CAMERA (camera), NULL);
+	g_return_val_if_fail (n_pixel_formats != NULL, NULL);
+
+	enumeration = arv_gc_get_node (camera->priv->genicam, "PixelFormat");
+	if (!ARV_IS_GC_ENUMERATION (enumeration))
+		return NULL;
+
+	*n_pixel_formats = arv_gc_node_get_n_childs (enumeration);
+	if (*n_pixel_formats == 0)
+		return NULL;
+
+	pixel_formats = g_new (ArvPixelFormat, *n_pixel_formats);
+	entries = arv_gc_enumeration_get_entries (ARV_GC_ENUMERATION (enumeration));
+	for (iter = entries, i = 0; iter != NULL; iter = iter->next, i++)
+		pixel_formats[i] = arv_gc_enum_entry_get_value (iter->data);
+
+	return pixel_formats;
+}
+
 /* Acquisition control */
 
 /**
@@ -342,6 +422,9 @@ arv_camera_set_frame_rate (ArvCamera *camera, double frame_rate)
 {
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 
+	if (frame_rate <= 0.0)
+		return;
+
 	switch (camera->priv->vendor) {
 		case ARV_CAMERA_VENDOR_BASLER:
 			arv_device_set_string_feature_value (camera->priv->device, "TriggerSelector",
@@ -454,7 +537,20 @@ arv_camera_set_exposure_time (ArvCamera *camera, double exposure_time_us)
 {
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 
-	arv_device_set_float_feature_value (camera->priv->device, "ExposureTimeAbs", exposure_time_us);
+	if (exposure_time_us <= 0)
+		return;
+
+	switch (camera->priv->series) {
+		case ARV_CAMERA_SERIES_BASLER_SCOUT:
+			arv_device_set_float_feature_value (camera->priv->device, "ExposureTimeBaseAbs",
+							    exposure_time_us);
+			arv_device_set_integer_feature_value (camera->priv->device, "ExposureTimeRaw", 1);
+			break;
+		case ARV_CAMERA_SERIES_BASLER_ACE:
+		default:
+			arv_device_set_float_feature_value (camera->priv->device, "ExposureTimeAbs", exposure_time_us);
+			break;
+	}
 }
 
 /**
@@ -471,6 +567,46 @@ arv_camera_get_exposure_time (ArvCamera *camera)
 	return arv_device_get_float_feature_value (camera->priv->device, "ExposureTimeAbs");
 }
 
+void
+arv_camera_get_exposure_time_bounds (ArvCamera *camera, double *min, double *max)
+{
+	gint64 int_min, int_max;
+
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	switch (camera->priv->series) {
+		case ARV_CAMERA_SERIES_BASLER_SCOUT:
+			arv_device_get_float_feature_bounds (camera->priv->device, "ExposureTimeBaseAbs", min, max);
+			break;
+		case ARV_CAMERA_SERIES_BASLER_ACE:
+			arv_device_get_integer_feature_bounds (camera->priv->device, "ExposureTimeRaw", &int_min, &int_max);
+			if (min != NULL)
+				*min = int_min;
+			if (max != NULL)
+				*max = int_max;
+			break;
+		default:
+			arv_device_get_float_feature_bounds (camera->priv->device, "ExposureTimeAbs", min, max);
+			break;
+	}
+}
+
+void
+arv_camera_set_exposure_time_auto (ArvCamera *camera, ArvAuto auto_mode)
+{
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	arv_device_set_string_feature_value (camera->priv->device, "ExposureAuto", arv_auto_to_string (auto_mode));
+}
+
+ArvAuto
+arv_camera_get_exposure_time_auto (ArvCamera *camera)
+{
+	g_return_val_if_fail (ARV_IS_CAMERA (camera), ARV_AUTO_OFF);
+
+	return arv_auto_from_string (arv_device_get_string_feature_value (camera->priv->device, "ExposureAuto"));
+}
+
 /* Analog control */
 
 /**
@@ -482,10 +618,13 @@ arv_camera_get_exposure_time (ArvCamera *camera)
  */
 
 void
-arv_camera_set_gain (ArvCamera *camera, gint64 gain)
+arv_camera_set_gain (ArvCamera *camera, gint gain)
 {
 	g_return_if_fail (ARV_IS_CAMERA (camera));
 
+	if (gain < 0)
+		return;
+
 	arv_device_set_integer_feature_value (camera->priv->device, "GainRaw", gain);
 }
 
@@ -495,7 +634,7 @@ arv_camera_set_gain (ArvCamera *camera, gint64 gain)
  * Return value: the current gain setting.
  */
 
-gint64
+gint
 arv_camera_get_gain (ArvCamera *camera)
 {
 	g_return_val_if_fail (ARV_IS_CAMERA (camera), 0.0);
@@ -503,6 +642,37 @@ arv_camera_get_gain (ArvCamera *camera)
 	return arv_device_get_integer_feature_value (camera->priv->device, "GainRaw");
 }
 
+void
+arv_camera_get_gain_bounds (ArvCamera *camera, gint *min, gint *max)
+{
+	gint64 min64, max64;
+
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	arv_device_get_integer_feature_bounds (camera->priv->device, "GainRaw", &min64, &max64);
+
+	if (min != NULL)
+		*min = min64;
+	if (max != NULL)
+		*max = max64;
+}
+
+void
+arv_camera_set_gain_auto (ArvCamera *camera, ArvAuto auto_mode)
+{
+	g_return_if_fail (ARV_IS_CAMERA (camera));
+
+	arv_device_set_string_feature_value (camera->priv->device, "GainAuto", arv_auto_to_string (auto_mode));
+}
+
+ArvAuto
+arv_camera_get_gain_auto (ArvCamera *camera)
+{
+	g_return_val_if_fail (ARV_IS_CAMERA (camera), ARV_AUTO_OFF);
+
+	return arv_auto_from_string (arv_device_get_string_feature_value (camera->priv->device, "GainAuto"));
+}
+
 /* Transport layer control */
 
 /**
@@ -553,9 +723,11 @@ arv_camera_new (const char *name)
 	ArvCamera *camera;
 	ArvDevice *device;
 	ArvCameraVendor vendor;
+	ArvCameraSeries series;
 	const char *vendor_name;
+	const char *model_name;
 
-	device = arv_new_device (name);
+	device = arv_open_device (name);
 
 	if (!ARV_IS_DEVICE (device))
 		return NULL;
@@ -565,14 +737,26 @@ arv_camera_new (const char *name)
 	camera->priv->genicam = arv_device_get_genicam (device);
 
 	vendor_name = arv_camera_get_vendor_name (camera);
-	if (g_strcmp0 (vendor_name, "Basler") == 0)
+	model_name = arv_camera_get_model_name (camera);
+
+	if (g_strcmp0 (vendor_name, "Basler") == 0) {
 		vendor = ARV_CAMERA_VENDOR_BASLER;
-	else if (g_strcmp0 (vendor_name, "Prosilica") == 0)
+		if (g_str_has_prefix (model_name, "acA"))
+			series = ARV_CAMERA_SERIES_BASLER_ACE;
+		else if (g_str_has_prefix (model_name, "scA"))
+			series = ARV_CAMERA_SERIES_BASLER_SCOUT;
+		else
+			series = ARV_CAMERA_SERIES_BASLER_OTHER;
+	} else if (g_strcmp0 (vendor_name, "Prosilica") == 0) {
 		vendor = ARV_CAMERA_VENDOR_PROSILICA;
-	else
+		series = ARV_CAMERA_SERIES_PROSILICA_OTHER;
+	} else {
 		vendor = ARV_CAMERA_VENDOR_UNKNOWN;
+		series = ARV_CAMERA_SERIES_UNKNOWN;
+	}
 
 	camera->priv->vendor = vendor;
+	camera->priv->series = series;
 
 	return camera;
 }
