@@ -139,9 +139,15 @@ arv_pixel_format_get_type (void)
 			{ ARV_PIXEL_FORMAT_MONO_8,
 			  "ARV_PIXEL_FORMAT_MONO_8",
 			  "mono-8" },
+			{ ARV_PIXEL_FORMAT_MONO_8_SIGNED,
+			  "ARV_PIXEL_FORMAT_MONO_8_SIGNED",
+			  "mono-8-signed" },
 			{ ARV_PIXEL_FORMAT_MONO_10,
 			  "ARV_PIXEL_FORMAT_MONO_10",
 			  "mono-10" },
+			{ ARV_PIXEL_FORMAT_MONO_10_PACKED,
+			  "ARV_PIXEL_FORMAT_MONO_10_PACKED",
+			  "mono-10-packed" },
 			{ ARV_PIXEL_FORMAT_MONO_12,
 			  "ARV_PIXEL_FORMAT_MONO_12",
 			  "mono-12" },
@@ -220,33 +226,48 @@ arv_pixel_format_get_type (void)
 			{ ARV_PIXEL_FORMAT_YUV_444_PACKED,
 			  "ARV_PIXEL_FORMAT_YUV_444_PACKED",
 			  "yuv-444-packed" },
-			{ ARV_PIXEL_FORMAT_BAYER_GR_12_PACKED,
-			  "ARV_PIXEL_FORMAT_BAYER_GR_12_PACKED",
-			  "bayer-gr-12-packed" },
-			{ ARV_PIXEL_FORMAT_BAYER_RG_12_PACKED,
-			  "ARV_PIXEL_FORMAT_BAYER_RG_12_PACKED",
-			  "bayer-rg-12-packed" },
-			{ ARV_PIXEL_FORMAT_BAYER_BG_12_PACKED,
-			  "ARV_PIXEL_FORMAT_BAYER_BG_12_PACKED",
-			  "bayer-bg-12-packed" },
-			{ ARV_PIXEL_FORMAT_BAYER_GB_12_PACKED,
-			  "ARV_PIXEL_FORMAT_BAYER_GB_12_PACKED",
-			  "bayer-gb-12-packed" },
+			{ ARV_PIXEL_FORMAT_RGB_8_PLANAR,
+			  "ARV_PIXEL_FORMAT_RGB_8_PLANAR",
+			  "rgb-8-planar" },
+			{ ARV_PIXEL_FORMAT_RGB_10_PLANAR,
+			  "ARV_PIXEL_FORMAT_RGB_10_PLANAR",
+			  "rgb-10-planar" },
+			{ ARV_PIXEL_FORMAT_RGB_12_PLANAR,
+			  "ARV_PIXEL_FORMAT_RGB_12_PLANAR",
+			  "rgb-12-planar" },
+			{ ARV_PIXEL_FORMAT_RGB_16_PLANAR,
+			  "ARV_PIXEL_FORMAT_RGB_16_PLANAR",
+			  "rgb-16-planar" },
 			{ ARV_PIXEL_FORMAT_YUV_422_YUYV_PACKED,
 			  "ARV_PIXEL_FORMAT_YUV_422_YUYV_PACKED",
 			  "yuv-422-yuyv-packed" },
-			{ ARV_PIXEL_FORMAT_BAYER_GR_16,
-			  "ARV_PIXEL_FORMAT_BAYER_GR_16",
-			  "bayer-gr-16" },
-			{ ARV_PIXEL_FORMAT_BAYER_RG_16,
-			  "ARV_PIXEL_FORMAT_BAYER_RG_16",
-			  "bayer-rg-16" },
-			{ ARV_PIXEL_FORMAT_BAYER_GB_16,
-			  "ARV_PIXEL_FORMAT_BAYER_GB_16",
-			  "bayer-gb-16" },
-			{ ARV_PIXEL_FORMAT_BAYER_BG_16,
-			  "ARV_PIXEL_FORMAT_BAYER_BG_16",
-			  "bayer-bg-16" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_GR_12_PACKED,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_GR_12_PACKED",
+			  "custom-bayer-gr-12-packed" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_RG_12_PACKED,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_RG_12_PACKED",
+			  "custom-bayer-rg-12-packed" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_GB_12_PACKED,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_GB_12_PACKED",
+			  "custom-bayer-gb-12-packed" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_BG_12_PACKED,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_BG_12_PACKED",
+			  "custom-bayer-bg-12-packed" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_YUV_422_YUYV_PACKED,
+			  "ARV_PIXEL_FORMAT_CUSTOM_YUV_422_YUYV_PACKED",
+			  "custom-yuv-422-yuyv-packed" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_GR_16,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_GR_16",
+			  "custom-bayer-gr-16" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_RG_16,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_RG_16",
+			  "custom-bayer-rg-16" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_GB_16,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_GB_16",
+			  "custom-bayer-gb-16" },
+			{ ARV_PIXEL_FORMAT_CUSTOM_BAYER_BG_16,
+			  "ARV_PIXEL_FORMAT_CUSTOM_BAYER_BG_16",
+			  "custom-bayer-bg-16" },
 			{ 0, NULL, NULL }
 		};
 		the_type = g_enum_register_static (
@@ -502,23 +523,23 @@ arv_gvsp_packet_type_get_type (void)
 #include "arvgvstream.h"
 
 GType
-arv_gv_stream_option_get_type (void)
+arv_gv_stream_socket_buffer_get_type (void)
 {
 	static GType the_type = 0;
 
 	if (the_type == 0)
 	{
 		static const GEnumValue values[] = {
-			{ ARV_GV_STREAM_OPTION_SOCKET_BUFFER_FIXED,
-			  "ARV_GV_STREAM_OPTION_SOCKET_BUFFER_FIXED",
+			{ ARV_GV_STREAM_SOCKET_BUFFER_FIXED,
+			  "ARV_GV_STREAM_SOCKET_BUFFER_FIXED",
 			  "fixed" },
-			{ ARV_GV_STREAM_OPTION_SOCKET_BUFFER_AUTO,
-			  "ARV_GV_STREAM_OPTION_SOCKET_BUFFER_AUTO",
+			{ ARV_GV_STREAM_SOCKET_BUFFER_AUTO,
+			  "ARV_GV_STREAM_SOCKET_BUFFER_AUTO",
 			  "auto" },
 			{ 0, NULL, NULL }
 		};
 		the_type = g_enum_register_static (
-				g_intern_static_string ("ArvGvStreamOption"),
+				g_intern_static_string ("ArvGvStreamSocketBuffer"),
 				values);
 	}
 	return the_type;
