@@ -61,8 +61,8 @@ GType arv_gvsp_packet_type_get_type	(void) G_GNUC_CONST;
 
 /* Enumerations from "arvgvstream.h" */
 
-#define ARV_TYPE_GV_STREAM_OPTION	(arv_gv_stream_option_get_type())
-GType arv_gv_stream_option_get_type	(void) G_GNUC_CONST;
+#define ARV_TYPE_GV_STREAM_SOCKET_BUFFER	(arv_gv_stream_socket_buffer_get_type())
+GType arv_gv_stream_socket_buffer_get_type	(void) G_GNUC_CONST;
 
 #define ARV_TYPE_GV_STREAM_PACKET_RESEND	(arv_gv_stream_packet_resend_get_type())
 GType arv_gv_stream_packet_resend_get_type	(void) G_GNUC_CONST;
